@@ -10,9 +10,21 @@ see <https://www.gnu.org/licenses/>.
 """
 import os
 
+#  https://stackoverflow.com/questions/1854/python-what-os-am-i-running-on
+from sys import platform as _platform
+
+if _platform == 'win32':
+    import ctypes
+    import qdarkstyle
+elif _platform == 'darwin':
+    import glob
+else:
+    raise Exception('Unsupported platform ',_platform)
+
 from PyQt5.QtCore import QTimer
 from PyQt5.QtWidgets import QApplication, QMainWindow
 from PyQt5.QtGui import QPixmap, QIcon
+from PyQt5.QtNetwork import QUdpSocket, QHostAddress
 
 from umirobot.shared_memory import UMIRobotSharedMemoryReceiver
 from umirobot.gui._umirobot_main_window_ui import Ui_MainWindow
@@ -37,7 +49,15 @@ class UMIRobotMainWindow(QMainWindow):
         self.pot_max = [None] * 6
         self.potentiometer_calibration_difference = 3.0
 
-        self.teleoperation_qd = [None] * 6;
+        self.potentiometer_qd = [None] * 6
+        self.socket_qd = [None] * 6
+
+        # TODO read socket info from config file
+        self.socket_ip = '127.0.0.1'
+        self.socket_port = 2222
+        self.socket = None
+        self.ui.lineedit_teleoperation_socket.setText(str(self.socket_ip + ":" + str(self.socket_port)))
+        self._connect_socket()
 
         # Fix robot figure not showing
         # https://stackoverflow.com/questions/33454555/bind-relative-image-path-to-py-file
@@ -83,6 +103,15 @@ class UMIRobotMainWindow(QMainWindow):
     def log(self, msg):
         self.ui.textedit_status.append(msg)
 
+    def _connect_socket(self):
+        self.socket = QUdpSocket()
+        self.socket.bind(QHostAddress(self.socket_ip), self.socket_port)
+        self.socket.readyRead.connect(self._socket_ready_read_callback)
+
+    def _socket_ready_read_callback(self):
+        # TODO Read datagram
+        pass
+
     def _get_manual_control_increment(self):
         return self.ui.slider_manual_control.value()
 
@@ -176,6 +205,8 @@ class UMIRobotMainWindow(QMainWindow):
 
         if current_is_open:
             self.is_open = True
+            self.ui.label_connection_status.setText(
+                "Connected to {}.".format(self.umi_robot_shared_memory_receiver.get_port()))
             try:
                 q = self.umi_robot_shared_memory_receiver.get_q()
                 self.ui.lineedit_q_0.setText(str(q[0]))
@@ -189,14 +220,15 @@ class UMIRobotMainWindow(QMainWindow):
 
                 if self._are_potentiometers_calibrated():
                     for i in range(0, 6):
-                        self.teleoperation_qd[i] = int(float(potentiometer_values[i] / (self.pot_max[i] - self.pot_min[i]))*120.0) - 60
+                        self.potentiometer_qd[i] = int(
+                            float(potentiometer_values[i] / (self.pot_max[i] - self.pot_min[i])) * 120.0) - 60
 
-                self.ui.lineedit_pot_0.setText(str(self.teleoperation_qd[0]))
-                self.ui.lineedit_pot_1.setText(str(self.teleoperation_qd[1]))
-                self.ui.lineedit_pot_2.setText(str(self.teleoperation_qd[2]))
-                self.ui.lineedit_pot_3.setText(str(self.teleoperation_qd[3]))
-                self.ui.lineedit_pot_4.setText(str(self.teleoperation_qd[4]))
-                self.ui.lineedit_pot_5.setText(str(self.teleoperation_qd[5]))
+                self.ui.lineedit_pot_0.setText(str(self.potentiometer_qd[0]))
+                self.ui.lineedit_pot_1.setText(str(self.potentiometer_qd[1]))
+                self.ui.lineedit_pot_2.setText(str(self.potentiometer_qd[2]))
+                self.ui.lineedit_pot_3.setText(str(self.potentiometer_qd[3]))
+                self.ui.lineedit_pot_4.setText(str(self.potentiometer_qd[4]))
+                self.ui.lineedit_pot_5.setText(str(self.potentiometer_qd[5]))
 
                 if self.pot_min[0] is not None:
                     self.ui.lineedit_pot_min_0.setText("{0:.3g} V".format(self.pot_min[0]))
@@ -227,7 +259,7 @@ class UMIRobotMainWindow(QMainWindow):
                 if self.ui.checkbox_teleoperation.isChecked():
                     if self._are_potentiometers_calibrated():
                         for i in range(0, 6):
-                            self.qd[i] = self.teleoperation_qd[i]
+                            self.qd[i] = self.potentiometer_qd[i]
 
                 if self.ui.checkbox_potentiometer_cal.isChecked():
                     for i in range(0, 6):
@@ -268,6 +300,7 @@ class UMIRobotMainWindow(QMainWindow):
                 self.log("Error::_timer_callback" + str(e))
         else:
             self.is_open = False
+            self.ui.label_connection_status.setText("Disconnected.")
 
     def _are_potentiometers_calibrated(self):
         if self.ui.radio_pot_0.isChecked() and \
@@ -288,20 +321,26 @@ class UMIRobotMainWindow(QMainWindow):
             return
         if current_selected_port != "":
             self.umi_robot_shared_memory_receiver.send_port(current_selected_port)
-            self.log("Info::Connecting to {}...".format(current_selected_port))
+            self.log("Info::Requesting connection to {}...".format(current_selected_port))
         else:
             self.log("Warning::No port selected.")
 
     def _button_refresh_port_clicked_callback(self):
         import serial.tools.list_ports
         self.ui.combobox_port.clear()
-        ports_list = serial.tools.list_ports.comports()
+        if _platform == 'win32':
+            ports_list = serial.tools.list_ports.comports()
+        elif _platform == 'darwin':
+            # https://stackoverflow.com/questions/22467683/listing-serial-ports-in-mac-os-x-and-python-3
+            ports_list = glob.glob('/dev/tty.*')
         for port in ports_list:
             self.ui.combobox_port.addItem(str(port.name))
             if "Arduino Uno" in port.description:
                 self.log("Info::Arduino Uno found at {}.".format(port.name))
         if len(ports_list) == 0:
             self.log("Warning::Updated ports but no ports available.")
+        else:
+            self.log("Info::Found {} serial ports.".format(len(ports_list)))
 
     @staticmethod
     def run(shared_memory_info, lock):
@@ -309,22 +348,22 @@ class UMIRobotMainWindow(QMainWindow):
         app = QApplication([])
         myapp = UMIRobotMainWindow(umi_robot_shared_memory_receiver)
 
-        # https://stackoverflow.com/questions/1551605/how-to-set-applications-taskbar-icon-in-windows-7
+        # Set app icon
         path = os.path.dirname(os.path.abspath(__file__))
         app.setWindowIcon(QIcon(os.path.join(path, 'icon.png')))
-        if os.name == 'nt':
-            import qdarkstyle
+
+        # Set theme for Windows and fix icon not showing on taskbar
+        if _platform == 'win32':
             myapp.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt5'))
-            import ctypes
-            myappid = 'utokyo.umirobot'
-            ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
+            # https://stackoverflow.com/questions/1551605/how-to-set-applications-taskbar-icon-in-windows-7
+            ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID('utokyo.umirobot')
 
         myapp.show()
 
         try:
             app.exec_()
         except Exception as e:
-            print("umirobot_main_window::run::Error::"+str(e))
+            print("umirobot_main_window::run::Error::" + str(e))
         except KeyboardInterrupt:
             print("umirobot_main_window::run::Info::Interrupted by user.")
 
